/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "About.h"

#include <glib/gi18n.h>

#define WIDTH 320
#define HEIGHT 200
#define LEFT 68
#define RIGHT 305
#define TOP 35
#define BOTTOM 160

#define IMAGE "about_bg.png"

// the one and only dialog
About* About::the_dialog=NULL; 


void
About::show_dialog()
{
    if(!the_dialog){
        the_dialog = new About();
        the_dialog->show(); 
    }
}

About::About()
 : Gtk::Window(Gtk::WINDOW_TOPLEVEL)
{
    is_running = false;
    textpos = 0;
    set_title(_("About RealTimeBattle Client"));
    set_resizable(false);
    add(m_EventBox);
    m_EventBox.add(m_DrawingArea);

    image = Gdk::Pixbuf::create_from_file(IMAGE);

    m_DrawingArea.set_size_request(WIDTH, HEIGHT);
    m_DrawingArea.signal_expose_event().connect(sigc::mem_fun(*this, &About::on_expose_event));
    m_EventBox.signal_button_press_event().connect(sigc::mem_fun(*this, &About::on_drawable_clicked));

    show_all_children();
}

About::~About()
{
    the_dialog = NULL;
}

bool
About::on_drawable_clicked(GdkEventButton * data)
{
    hide();
    delete this;
    return false;
}

bool
About::update_text()
{
    pixmap = Gdk::Pixmap::create(m_DrawingArea.get_window(), WIDTH, HEIGHT, -1);
    image->render_to_drawable(pixmap, get_style()->get_black_gc(), 0, 0, 0, 0, image->get_width(), image->get_height(), Gdk::RGB_DITHER_NONE, 0, 0);

    int x, y;
    textpos++;
    pangolayout->get_pixel_size(x, y);
    if (textpos > BOTTOM - TOP + y + 10)
	textpos = 0;

    pixmap->draw_layout(clip_gc, LEFT, BOTTOM - textpos, pangolayout);

    Glib::RefPtr < Gdk::Window > win = m_DrawingArea.get_window();
    win->draw_drawable(m_DrawingArea.get_style()->get_black_gc(), pixmap, 0, 0, 0, 0);

    sigc::connection conn = Glib::signal_timeout().connect(sigc::mem_fun(*this, &About::update_text), int (50));
    return false;
}

bool
About::on_expose_event(GdkEventExpose *)
{
    if (is_running == false) {
	pixmap = Gdk::Pixmap::create(m_DrawingArea.get_window(), WIDTH, HEIGHT, -1);
	image->render_to_drawable(pixmap, get_style()->get_black_gc(), 0, 0, 0, 0, image->get_width(), image->get_height(), Gdk::RGB_DITHER_NONE, 0, 0);

	clip_gc = Gdk::GC::create(pixmap);

	Gdk::Rectangle clip_rect(LEFT, TOP, RIGHT - LEFT, BOTTOM - TOP);
	clip_gc->set_clip_rectangle(clip_rect);

	Glib::ustring content = "<b>";
	content += _("Brought to you by");
	content += "</b>\nVolker Ströbel\nDem Sandmännchen\nund der Zahnfee\n \n";

	content += "<b>";
	content += _("Dokumentation");
	content += "</b>\näm... Viele Leutz\n \n";

	content += "<b>";
	content += _("Testing");
	content += "</b>\nAlle beteiligte\n \n";
	content += "<b>";
	content += _("Stimmungsschwankungen\nund Wutausbrüche");
	content += "</b>\nVolker Ströbel\n \n";
	content += "<b>";

	content += _("Wichtige Helfer");
	content += "</b>\nBier\nSchlaf\nund Bier\n \n";

	content += "<b>";
	content += _("3D Zwischensequenzen");
	content += "</b>\nGibts nicht\n \n";

	content += "<b>";
	content += _("Briefmarkenkleben");
	content += "</b>\nGabs auch nicht\n \n";

	content += "<b>";
	content += _("Unverständliches\nNuscheln");
	content += "</b>\nBrrrBlubbBlubb\n \n";

	content += "<b>";
	content += _("Volker Ströbels\nseelischer Frieden\nzerstört durch");
	content += "</b>\nNüchterne Menschen\nAngela Merkel\nund Smørebrød";

	pangolayout = create_pango_layout("");
	Pango::FontDescription font;
	font.set_family("sans");
	font.set_style(Pango::STYLE_NORMAL);
	font.set_weight(Pango::WEIGHT_NORMAL);
	font.set_size(Pango::SCALE * 10);
	pangolayout->set_font_description(font);

	pangolayout->set_markup(content);

	pixmap->draw_layout(clip_gc, LEFT, BOTTOM - textpos, pangolayout);

	is_running = true;
	sigc::connection conn = Glib::signal_timeout().connect(sigc::mem_fun(*this, &About::update_text), int (25));
    }
    Glib::RefPtr < Gdk::Window > win = m_DrawingArea.get_window();
    win->draw_drawable(m_DrawingArea.get_style()->get_black_gc(), pixmap, 0, 0, 0, 0);

    return false;
}


bool
About::on_delete_event(GdkEventAny * event)
{
    hide();
    delete this;
    return false;
}
