/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "math.h"
#include "signal.h"

#include <string>
#include <list>
#include <sstream>
using namespace std;

#include "Arena.h"

#include "MainWindow.h"
#include "ArenaView.h"
#include "MessageView.h"
extern MainWindow *the_gui;
extern ArenaView *the_arenaview;
extern ScoreView *the_scoreview;
extern MessageView *the_messageview;

#include "CoreInterface.h"


#define sgn(a) ((a) > 0 ? 1 : -1)
//#define CORENAME Glib::get_current_dir()+="/testcore"
#define CORENAME Glib::get_current_dir()+="/rtbcore"

stringstream stream;
Glib::ustring line;
string line2;
Glib::RefPtr < Glib::IOChannel > core_in, error_in;


Arena::Arena()
{
}

Arena::~Arena()
{
}

bool
Arena::IOCallback(Glib::IOCondition condition)
{
    if (condition == Glib::IO_IN) {
        try {
            core_in->read_line(line);
            stream.str(line);
            
            char result;       
            result = parse_line();
            if (result == 'T') {
                the_arenaview->parse_turn();
            }  
        } catch(Glib::Error e) {
            cout << "IOCallBAck:: Error while reading line" << endl;
        }
        return true;
    } else if (condition == Glib::IO_HUP) {
        cout << "Core? Core??? :(" << endl;
        return false;
    } else {
        cout << "D'oh" << endl;
        return false;
    }
}

bool
Arena::IOErrorCallback(Glib::IOCondition condition)
{
    if (condition == Glib::IO_IN) {
        Glib::ustring error;
        cerr << "Core Error!!!" << endl;
        error_in->read_line(error);
        cerr << "Core Error:" << error << ends;
        return true;
    } else if (condition == Glib::IO_HUP) {
        return false;
    } else {
        return false;
    }
}


bool
Arena::FILECallback()
{

    char temp[256];
    char result='0';
    do {
        try{
            file->getline(temp, 255);
            line = temp;
            line += '\n';
            stream.str(line);
            result = parse_line();
        }catch(const Glib::Error &e){
            //this is normaly an error in the character set conversion
            //todo: implement fallback method 
            //cerr << e.what() << endl;
        }
    } while ((!file->eof()) && (result != 'T'));

    if (result == 'T') {
        the_arenaview->parse_turn();
        if (timescale > 0.0)
            sigc::connection conn = Glib::signal_timeout().connect(sigc::mem_fun(*this, &Arena::FILECallback), int (timescale * 1000));
        else
            sigc::connection conn = Glib::signal_timeout().connect(sigc::mem_fun(*this, &Arena::FILECallback), 50);
    }
    return false;
}


bool
Arena::new_tournament(const char *name, TournamentType ttype)
{
    switch (ttype) {
    case RTB_LOGFILE:
	{
	    file = new fstream(name);
	    if (file->is_open()) {
		//ein = file;
		/*
		 *      Clear old arena stuff
		 *      IMPORTANT!!! Delete Container elements to prevent memleaks!!! 
		 */
		//the_scoreview->clear();
		//the_messageview->clear();
		shapes.clear();
		robots.clear();
		mines.clear();
		cookies.clear();
		shoots.clear();
		last_tournament_time = 0.0;
		timescale = 0.0;
		this->ttype = RTB_LOGFILE;

		status = PARSER_START;
		sigc::connection conn = Glib::signal_timeout().connect(sigc::mem_fun(*this, &Arena::FILECallback), 100);
		return true;
	    } else
		return false;
	}
	break;
    
    case RTB_CORE:
	{
	    /*
	     *      Clear old arena stuff
	     *      IMPORTANT!!! Delete Container elements to prevent memleaks!!! 
	     */
	   // the_scoreview->clear();
	   // the_messageview->clear();
	    shapes.clear();
	    robots.clear();
	    mines.clear();
	    cookies.clear();
	    shoots.clear();
	    last_tournament_time = 0.0;
	    timescale = 0.0;

	    this->ttype = RTB_CORE;

	    //error_in = Glib::IOChannel::create_from_fd(the_iface.socket);
	    //Glib::signal_io().connect(sigc::mem_fun(*this, &Arena::IOErrorCallback), error_in, Glib::IO_IN | Glib::IO_HUP);

	    core_in = Glib::IOChannel::create_from_fd(the_iface.get_socket());
        
        core_in->set_buffered(true);
        core_in->set_buffer_size(8192);
        
	    string charset;

	    //Robot Messages with illegal characters cannot be converted
	    //Therefor we should use UTF-8 for communication with the core
	    //But the core curently uses the local charset :(

        //	    Glib::get_charset(charset);
        //	    cout << "SyS Charset:" << charset << endl;
        //	    core_in->set_encoding(charset);

	    const Glib::RefPtr < Glib::IOSource > io_source = Glib::IOSource::create(core_in, Glib::IO_IN | Glib::IO_HUP);
	    io_source->set_priority(Glib::PRIORITY_LOW);
	    io_source->connect(sigc::mem_fun(*this, &Arena::IOCallback));
	    io_source->attach(Glib::MainContext::get_default());

	    status = PARSER_START;

	    return true;
	}
	break;
    
    default:
        return false;
    }
}

bool
Arena::parse_poly(bool polystart)
{
    if (polystart == true) {
	stream >> poly.bounce >> poly.hard >> poly.thick >> poly.startpos >> poly.direction;

	poly.currentpos.x = poly.startpos.x;
	poly.currentpos.y = poly.startpos.y;

	poly.direction.normalize();

    } else {
	char temp[256];
	double &thick = poly.thick;
	Vector2D & direction = poly.direction;
	Vector2D & startpos = poly.startpos;
	Vector2D & currentpos = poly.currentpos;

	char c;
	double len, angle, start_angle, end_angle, radius;
	bool finish = false;

	stream >> c;
	if (c == 'A') {
	    stream >> c;
	    switch (c) {
	    case 'A':
		{
		    stream >> angle >> radius;

		    Vector2D center, endpos;
		    Vector2D arcdirection1(direction.x, direction.y);
		    arcdirection1.rotate(-90 * sgn(angle) * (M_PI / 180.0));

		    Vector2D arcdirection2(arcdirection1.x, arcdirection1.y);
		    arcdirection2.rotate((180 - angle) * (M_PI / 180.0));

		    center.x = currentpos.x + radius * arcdirection1.x;
		    center.y = currentpos.y + radius * arcdirection1.y;
		    endpos.x = center.x + radius * arcdirection2.x;
		    endpos.y = center.y + radius * arcdirection2.y;

		    start_angle = atan2(currentpos.y - center.y, currentpos.x - center.x);
		    end_angle = atan2(currentpos.y - endpos.y, currentpos.x - endpos.x);

		    shapes.push_back(new Arc(center.x, center.y, radius, start_angle * 180 / M_PI, -1 * angle, thick));
		    currentpos.x = endpos.x;
		    currentpos.y = endpos.y;

		    direction.rotate(-angle * M_PI / 180.0);

		}
		break;

	    case 'T':
		{
		    stream >> angle;
		    direction.rotate(-angle * M_PI / 180.0);
		}
		break;

	    case 'L':
		{
		    stream >> len;
		    shapes.push_back(new Line(currentpos.x, currentpos.y, currentpos.x + len * direction.x, currentpos.y + len * direction.y, thick));
		    currentpos.x = currentpos.x + len * direction.x;
		    currentpos.y = currentpos.y + len * direction.y;
		}
		break;
	    case 'C':
		shapes.push_back(new Line(currentpos.x, currentpos.y, startpos.x, startpos.y, thick));
		finish = true;
		break;

	    case 'Q':		// quit
		finish = true;
		break;

	    default:
		finish = true;
		break;
	    }
	    stream.getline(temp, 255);
	} else {
	    finish = true;
	    stream.getline(temp, 255);
	}
	return finish;
    }
    return false;
}

char
Arena::parse_line()
{
    if (status == PARSER_POLY){
        if (parse_poly() == true)
            status = PARSER_START;
        return 'A';
    } else {
        
	char temp[256];
	char first_letter;
	string op;         
	stream >> first_letter;
        
	switch (first_letter) {
	case 'A':
	    stream >> op;
	    if (op == "scale") {
		stream >> scale;
		cout << op << " : " << scale << endl;
	    } else if (op == "boundary") {
		stream >> boundary.x1 >> boundary.y1 >> boundary.x2 >> boundary.y2;

		//cout << op << " : " << boundary.x1 << " " << boundary.y1 << " " << boundary.x2 << " " << boundary.y2 << " " << endl;
	    } else if (op == "line") {
		double bounce, hard, thick, x1, y1, x2, y2;
		stream >> bounce >> hard >> thick >> x1 >> y1 >> x2 >> y2;
		shapes.push_back(new Line(x1, y1, x2, y2, thick));

	    } else if (op == "circle") {
		double bounce, hard, x, y, radius;
		stream >> bounce >> hard >> x >> y >> radius;

		shapes.push_back(new Circle(x, y, radius));

	    } else if (op == "inner_circle") {
		double bounce, hard, x, y, radius;
		stream >> bounce >> hard >> x >> y >> radius;

		shapes.push_back(new Circle(x, y, radius, false));

	    } else if (op == "polygon") {
		double bounce, hard, thick, x1, y1;
		double vertices;

		stream >> bounce >> hard >> thick >> vertices;
		std::vector < Vector2D > points;
		for (int i = 0; i < vertices; i++) {
		    stream >> x1 >> y1;
		    points.push_back(Vector2D(x1, y1));

		}
		shapes.push_back(new Polygon(points, thick));

	    } else if (op == "closed_polygon") {
		double bounce, hard, thick, x1, y1, start_x, start_y;
		double vertices;

		stream >> bounce >> hard >> thick >> vertices;

		std::vector < Vector2D > points;
		stream >> start_x >> start_y;
		points.push_back(Vector2D(start_x, start_y));
		for (int i = 1; i < vertices; i++) {
		    stream >> x1 >> y1;
		    points.push_back(Vector2D(x1, y1));
		}
		points.push_back(Vector2D(start_x, start_y));
		shapes.push_back(new Polygon(points, thick));

	    } else if (op == "poly_curve") {

		parse_poly(true);
		status = PARSER_POLY;

	    } else if (op == "teleport") {
		double x, y, radius;
		stream >> x >> y >> radius >> radius;
		shapes.push_back(new Circle(x, y, 1.0, true, 0x0000ff));

	    } else
		cout << op << endl;

	    stream.getline(temp, 255);

	    break;

	case 'P':
	    {
		int robot;
		char message[256];

		stream >> robot;
		stream.get(message, 255);

		/*if (robot > 0) {
		    std::map < int, Robot * >::iterator it;
		    it = robots.find(robot);
		    if (it->second)
                (it->second)->message(message);
		} else
		    the_messageview->add_message("RTB Core", message); */

		stream.getline(temp, 255);

	    }
	    break;

	case 'L':
	    {
		int robot;
		long color;
		char name[256];

		stream >> robot >> hex >> color;
		stream.get(name, 255);
		//the_scoreview->clear();
		//if (robot==0) the_scoreview->clear();
		robots[robot] = new Robot(robot, color, name);
		//cout << "???: " << robot << endl;
		stream.getline(temp, 255);

	    }
	    break;

	case 'R':
	    {
		int robot;
		double x, y, direction, canon, radar, energy;

		stream >> robot >> x >> y >> direction >> canon >> radar >> energy;

		std::map < int, Robot * >::iterator it;
		it = robots.find(robot);
		if (it->second) {
		    (it->second)->set_position(x, y, direction, canon, radar);
		    (it->second)->update_robot(energy);
		}

		stream.getline(temp, 255);
	    }
	    break;

	case 'M':
	    {
		int mine;
		double x, y;
		stream >> mine >> x >> y;
		mines[mine] = new MovingShape(x, y, 0xff0000);
		stream.getline(temp, 255);
	    }
	    break;

	case 'C':
	    {
		int cookie;
		double x, y;
		stream >> cookie >> x >> y;
		cookies[cookie] = new MovingShape(x, y, 0x00ff00);
		stream.getline(temp, 255);
	    }
	    break;

	case 'S':
	    {
		int shot;
		double x, y;
		Vector2D direction;

		stream >> shot >> x >> y >> direction;

		shoots[shot] = new Shot(x, y, direction);
		stream.getline(temp, 255);
	    }
	    break;

	case 'D':
	    {
		char what;
		stream >> what;
		if (what == 'R') {
		    int robot, place;
		    double points;
		    if (!messageview_cleared) {
		    	messageview_cleared = true;			  
		        the_messageview->clear();
		    }
		    
		    stream >> robot >> points >> place;
		   // cout << robot << endl;
		    if (n_robots <= robot) n_robots = robot+1;
		    std::map < int, Robot * >::iterator it;
		    it = robots.find(robot);
		    if (it->second) {
			(it->second)->update_robot(500.0, place);
			(it->second)->update_points(points);
		    }
		    (it->second)->kill();
		    if (second) {
		    //	cout << "N_robots: " << n_robots << endl;
		    	for (int i = 0; i < n_robots; i++) {
			std::map < int, Robot * >::iterator it2;
		  			it2 = robots.find(i);
		    		if (it2->second) {
					(it2->second)->update_message_view();
		    		}
				//cout << i << endl;
				//if (robots[i] != NULL) robots[i]->update_message_view();
			}
		    }
		    if (!second) second = true;
		} else if (what == 'M') {
		    int mine;
		    stream >> mine;
		    std::map < int, MovingShape * >::iterator it;
		    it = mines.find(mine);
		    if (it->second) {
			(it->second)->remove_shape();
			mines.erase(it);
		    }
		} else if (what == 'C') {
		    int cookie;
		    stream >> cookie;
		    std::map < int, MovingShape * >::iterator it;
		    it = cookies.find(cookie);
		    if (it->second) {
			(it->second)->remove_shape();
			cookies.erase(it);
		    }
		} else if (what == 'S') {
		    int shot;
		    stream >> shot;
		    std::map < int, Shot * >::iterator it;
		    it = shoots.find(shot);
		    if (it->second) {
			(it->second)->remove_shape();
			shoots.erase(it);
		    }
		} else
		    cout << "D???: " << what << endl;

		stream.getline(temp, 255);
	    }
	    break;

	case 'T':
	    double time;
	    stream >> time;
	    timescale = time - last_tournament_time;
	    last_tournament_time = time;
	    the_gui->set_progress(time, timeout);
        stream.getline(temp, 255);
	    break;

	case 'G':
	    double seq, game;
	    stream >> seq >> game;

	    messageview_cleared = false;
	    second = false;
	    shapes.clear();
	    mines.clear();
	    cookies.clear();
	    shoots.clear();
	    the_scoreview->clear();
	  /*   
	    //Clear Robot place
	    {
		std::map < int, Robot * >::iterator it;
		for (it = robots.begin(); it != robots.end(); it++) {
		    (it->second)->update_robot(0.0, 0);
		}
		
		
		
	    } */

	    last_tournament_time = 0.0;
	    //timescale=0.25;
	    the_arenaview->draw_everything();
	    the_gui->set_gamesequence(int (game), int (seq), games, seqs);

	    stream.getline(temp, 255);
	    break;

	case 'H':

	    stream >> games >> robots_seq >> seqs;

	    stream.getline(temp, 255);
	    n_robots = 0;
	    robots.clear();
	    
	    break;

	case 'O':
	    stream.getline(temp, 255, ':');
	    op = temp;
        if (op == "Timeout [s]") {
            stream >> timeout;
	    }
	    stream.getline(temp, 255);
	    break;

	default:
	    stream.getline(temp, 255);
	    cout << "???: " << temp << endl;
	    break;
	}

	return first_letter;
    }
}
