/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _ARENA_H_
#define _ARENA_H_

#include <iostream>
#include <fstream>
#include <map>
#include <vector>

using namespace std;

#include "Vector2D.h"
#include "Shapes.h"
#include "Robot.h"

enum TournamentType{
    RTB_LOGFILE,
    RTB_CORE
};


class Arena
{
public:
    Arena();
    virtual ~ Arena();

    bool new_tournament(const char *name, TournamentType ttype);

    char parse_line();

    double scale;
    struct{
        double x1, y1, x2, y2;
    }boundary;
     
    std::vector < Shape * >shapes;
    std::map < int, Robot * >robots;
    std::map < int, MovingShape * >mines;
    std::map < int, MovingShape * >cookies;
    std::map < int, Shot * >shoots;
    double timescale;
    double last_tournament_time;
    TournamentType ttype;

protected:

    enum ParserState {
        PARSER_START,
        PARSER_RUNNING,
        PARSER_POLY
    };

    struct poly_struct{
        double thick, bounce, hard;
        Vector2D direction;
        Vector2D startpos;
        Vector2D currentpos;
    }poly;

    virtual bool IOCallback(Glib::IOCondition condition);
    virtual bool IOErrorCallback(Glib::IOCondition condition);
    virtual bool FILECallback();
    bool parse_poly(bool polystart = false);

    ParserState status;
    int games, robots_seq, seqs, n_robots;
    double timeout;
    bool messageview_cleared, second;
    fstream *file;
    pid_t pid;
    int fd[2];

};

#endif // _ARENA_H_
