/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ArenaDrawingArea.h"
#include "ArenaView.h"

#define GDK_360_DEGREES 23040	// 64 * 360 degrees


ArenaDrawingArea::ArenaDrawingArea(ArenaView * parent)
{
    this->parent = parent;
}


ArenaDrawingArea::~ArenaDrawingArea()
{
    // TODO: put destructor code here
}

Gdk::Color ArenaDrawingArea::make_gdk_color(const long col)
{

    Gdk::Color color;
    Glib::RefPtr < Gdk::Colormap > cmap = get_default_colormap();

    color.set_red(((col & 0xff0000) >> 16) * 0x101);
    color.set_green(((col & 0x00ff00) >> 8) * 0x101);
    color.set_blue((col & 0x0000ff) * 0x101);

    cmap->alloc_color(color);

    return color;
}

void ArenaDrawingArea::clear()
{
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(0xffffff));
    win->draw_rectangle(gc, true, 0, 0, get_width(), get_height());

}

void ArenaDrawingArea::draw_startupmessage(char *message)
{
    int x, y;
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(0x000000));

    Glib::RefPtr < Pango::Layout > pangolayout = create_pango_layout(message);

    Pango::FontDescription font;
    font.set_style(Pango::STYLE_NORMAL);
    font.set_weight(Pango::WEIGHT_HEAVY);

    font.set_size(Pango::SCALE * 25);

    pangolayout->set_font_description(font);
    pangolayout->get_pixel_size(x, y);
    x = (get_width() - x) / 2;
    y = (get_height() - y) / 2;


    win->draw_layout(gc, x, y, pangolayout);
}

void ArenaDrawingArea::draw_line(double x1, double y1, double x2, double y2, double thickness, long color)
{
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(color));

    x1 = (x1 + x_offset) * scale;
    y1 = ((y1 * -1) + y_offset) * scale;
    x2 = (x2 + x_offset) * scale;
    y2 = ((y2 * -1) + y_offset) * scale;

    thickness *= scale;

    if (thickness < 1.0)
	thickness = 1.0;

    gc->set_line_attributes(int (thickness), Gdk::LINE_SOLID, Gdk::CAP_NOT_LAST, Gdk::JOIN_MITER);
    win->draw_line(gc, int (x1), int (y1), int (x2), int (y2));
}

void ArenaDrawingArea::draw_lines(std::vector < Vector2D > points, double thickness, long color)
{
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(color));

    std::vector < Gdk::Point > gpoints;
    std::vector < Vector2D >::iterator it;

    for (it = points.begin(); it < points.end(); it++) {
	gpoints.push_back(Gdk::Point(int ((it->x + x_offset) * scale), int (((it->y * -1) + y_offset) * scale)));
    }

    thickness *= scale;
    if (thickness < 1.0)
	thickness = 1.0;

    gc->set_line_attributes(int (thickness), Gdk::LINE_SOLID, Gdk::CAP_ROUND, Gdk::JOIN_ROUND);
    win->draw_lines(gc, gpoints);
}

void ArenaDrawingArea::draw_circle(double x, double y, double radius, long color, const bool filled, double line_width)
{
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(color));

    x = (x + x_offset) * scale;
    y = ((y * -1) + y_offset) * scale;

    double r;
    if ((r = radius * scale) > 1.0) {
	if (line_width <= 0)
	    line_width = 1;
	else {
	    line_width = int (line_width * scale);
	    r = r + (line_width / 2);
	}
	gc->set_line_attributes(int (line_width), Gdk::LINE_SOLID, Gdk::CAP_NOT_LAST, Gdk::JOIN_MITER);
	win->draw_arc(gc, filled, int (x - r), int (y - r), (int) (r * 2.0 + 0.5), (int) (r * 2.0 + 0.5), 0, GDK_360_DEGREES);
    } else {
	win->draw_point(gc, int (x), int (y));
    }
}

void ArenaDrawingArea::draw_arc(double x, double y, double radius, double angle1, double angle2, double line_width, long color)
{
    Glib::RefPtr < Gdk::Window > win = get_window();
    Glib::RefPtr < Gdk::GC > gc = Gdk::GC::create(win);
    gc->set_foreground(make_gdk_color(color));

    x = (x + x_offset) * scale;
    y = ((y * -1) + y_offset) * scale;

    double r;
    if ((r = radius * scale) > 1.0) {
	if (line_width <= 0)
	    line_width = 1;
	else {
	    line_width = int (line_width * scale);
	    //      r= r + (line_width / 2);
	}
	gc->set_line_attributes(int (line_width), Gdk::LINE_SOLID, Gdk::CAP_ROUND, Gdk::JOIN_MITER);
	win->draw_arc(gc, false, int (x - r), int (y - r), (int) (r * 2.0 + 0.5), (int) (r * 2.0 + 0.5), int (angle1 * 64), int (angle2 * 64));
    } else {
	win->draw_point(gc, int (x), int (y));
    }
}

void ArenaDrawingArea::set_scale(double scale, int width, int height)
{
    this->scale = scale;

    if ((width != area_width) || (height != area_height)) {
	area_width = width;
	area_height = height;
	set_size_request(width, height);
    }
}

void ArenaDrawingArea::set_offset(double x, double y)
{
    x_offset = x;
    y_offset = y;
}

bool ArenaDrawingArea::on_expose_event(GdkEventExpose *)
{
    parent->draw_everything();
    return true;
}
