/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ArenaView.h"
#include "ArenaDrawingArea.h"
#include "Arena.h"

#include <glib/gi18n.h>
#include <gtkmm/stock.h>
#include <iostream>
#include <string>
using namespace std;

Arena the_arena;

ArenaView::ArenaView()
{
    is_started = false;
    is_running = false;
    zoom = 1.0;
    
    set_policy(Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
    set_size_request(500, 400);
    set_border_width(12);
    set_shadow_type(Gtk::SHADOW_IN);

    DrawingArea = manage(new ArenaDrawingArea(this));
    add(*DrawingArea);
    DrawingArea->set_size_request(400, 300);
    
    m_refActionGroup = Gtk::ActionGroup::create();
    m_refActionGroup->add(Gtk::Action::create("ZoomIn", Gtk::Stock::ZOOM_IN), sigc::mem_fun(*this, &ArenaView::zoom_in));
    m_refActionGroup->add(Gtk::Action::create("ZoomOut", Gtk::Stock::ZOOM_OUT), sigc::mem_fun(*this, &ArenaView::zoom_out));
    m_refActionGroup->add(Gtk::Action::create("Zoom100", Gtk::Stock::ZOOM_100), sigc::mem_fun(*this, &ArenaView::no_zoom)); 

    m_refActionGroup->set_sensitive(false);
    update_actions();
}

ArenaView::~ArenaView()
{
    // TODO: put destructor code here
}


bool
ArenaView::start(const char *name, TournamentType ttype)
{
    if(the_arena.new_tournament(name, ttype)) {
	is_started = true;
	is_running = true;
        
    m_refActionGroup->set_sensitive(true);
    update_actions();
        
	DrawingArea->clear();

	//      do{
	//              result = the_arena.parse_line();
	//      } while (result!='T' && result!='?');

	DrawingArea->set_offset(the_arena.boundary.x1 * (-1), the_arena.boundary.y1 * (-1));
	update_scale();
	draw_shapes();
	//      do{
	//              result = the_arena.parse_line();
	//      } while (result!='T' && result!='?');
	draw_robots();
	DrawingArea->draw_startupmessage("Start Tournament");
	startmessage = true;

	//      if (the_arena.ttype == rtb_file)        
	//              SigC::Connection conn = Glib::signal_timeout().connect( SigC::slot(*this, &ArenaView::parse_turn) , 50);
	//      else parse_turn();
	return true;
    } else
	return false;
}

void ArenaView::pause()
{
    if (is_running == true)
	is_running = false;
    else
	is_running = true;
}


bool ArenaView::parse_turn()
{

    if (is_running == true) {
	//if (the_arena.ttype == rtb_file){
	//      do{
	//              result = the_arena.parse_line();
	//      } while (result!='T' && result!='?');
	//}

	if ((the_arena.last_tournament_time > 3) && (startmessage == true)) {
	    startmessage = false;
	    DrawingArea->clear();
	}
	DrawingArea->set_offset(the_arena.boundary.x1 * (-1), the_arena.boundary.y1 * (-1));
	update_scale();
	draw_robots();
	draw_shapes();

	//update shoots
	{
	    std::map < int, Shot * >::iterator it;
	    for (it = the_arena.shoots.begin(); it != the_arena.shoots.end(); it++)
		(it->second)->update_shape();
	}

    }
    //if (the_arena.ttype == rtb_file)
    //      SigC::Connection conn = Glib::signal_timeout().connect( SigC::slot(*this, &ArenaView::parse_turn) , int(the_arena.timescale*1000));
    //else          
    //      SigC::Connection conn = Glib::signal_timeout().connect( SigC::slot(*this, &ArenaView::parse_turn) ,int(10));
    return false;
}

void ArenaView::draw_robots()
{
    //Draw Robots
    {
	std::map < int, Robot * >::iterator it;
	for (it = the_arena.robots.begin(); it != the_arena.robots.end(); it++) {
	    (it->second)->draw_robot(DrawingArea);
	}
    }

    //Draw mines
    {
	std::map < int, MovingShape * >::iterator it;
	for (it = the_arena.mines.begin(); it != the_arena.mines.end(); it++) {
	    (it->second)->draw_shape(DrawingArea);
	}
    }

    //Draw Cookies
    {
	std::map < int, MovingShape * >::iterator it;
	for (it = the_arena.cookies.begin(); it != the_arena.cookies.end(); it++) {
	    (it->second)->draw_shape(DrawingArea);
	}
    }

    //Draw Shoots
    {
	std::map < int, Shot * >::iterator it;
	for (it = the_arena.shoots.begin(); it != the_arena.shoots.end(); it++) {
	    (it->second)->draw_shape(DrawingArea);
	}
    }


}

void ArenaView::draw_shapes()
{
    std::vector < Shape * >::iterator it;
    for (it = the_arena.shapes.begin(); it < the_arena.shapes.end(); it++) {
	(*it)->draw_shape(DrawingArea);
    }
}

void ArenaView::update_scale()
{
    int width = get_width() - 48;
    int height = get_height() - 48;

    double scale_x = width / ((the_arena.boundary.x1 * (-1)) + the_arena.boundary.x2);
    double scale_y = height / ((the_arena.boundary.y1 * (-1)) + the_arena.boundary.y2);

    if (scale_x < scale_y) {
	DrawingArea->set_scale(scale_x * zoom, int (width * zoom), int (height * zoom));
    } else
	DrawingArea->set_scale(scale_y * zoom, int (width * zoom), int (height * zoom));
}

void
ArenaView::draw_everything()
{
    if (is_started == true) {
	update_scale();
	DrawingArea->clear();
	draw_shapes();
	draw_robots();
	if (startmessage == true)
	    DrawingArea->draw_startupmessage("Start Tournament");
    }
}

void
ArenaView::no_zoom()
{
    zoom = 1.0;
    update_actions();
    draw_everything();
}

void
ArenaView::zoom_out()
{
    zoom -= 0.25;
    if (zoom < 1.0)
	zoom = 1.0;
    update_actions();
    draw_everything();
}

void
ArenaView::zoom_in()
{
    zoom += 0.25;
    update_actions();
    draw_everything();
}

void 
ArenaView::update_actions()
{
    Glib::RefPtr<Gtk::Action> zoomout = 
        m_refActionGroup->get_action("ZoomOut");
    Glib::RefPtr<Gtk::Action> zoom100 = 
        m_refActionGroup->get_action("Zoom100");
    
    if(zoom<=1.0){
        if(zoomout) zoomout->set_sensitive(false);
        if(zoom100) zoom100->set_sensitive(false);       
    }else{
        if(zoomout) zoomout->set_sensitive(true);
        if(zoom100) zoom100->set_sensitive(true);  
    }    
}
