/* 
 * RealTimeBattle Client
 * Copyright (C) 2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "CoreInterface.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>


#include <iostream>
#include <glib/gi18n.h>
#include <gtkmm/stock.h>
#include <gtkmm/label.h>
#include <gtkmm/box.h>
#include <gtkmm/sizegroup.h>


using namespace std;

CoreInterface the_iface;

CoreInterface::CoreInterface()
 : connected(false)
{
}

CoreInterface::~CoreInterface()
{

}

void
CoreInterface::connect()
{   
    if(connected){
        return;
    }
    
    ConnectDialog dialog;
       
    if(dialog.run()!=Gtk::RESPONSE_OK){
        return;
    }   
    
    int rc;
    struct sockaddr_in localAddr, servAddr;
    struct hostent *h;   
    
    h = gethostbyname(dialog.get_server().c_str());
    if(h==NULL){
        cerr << "unknown host: " << dialog.get_server() << endl;
        return;
    }

    servAddr.sin_family = h->h_addrtype;
    memcpy((char *) &servAddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
    servAddr.sin_port = htons(dialog.get_port());

    // create socket
    socket = ::socket(AF_INET, SOCK_STREAM, 0);
    if(socket<0){
        perror("cannot open socket ");
        return;
    }

    // bind any port number
    localAddr.sin_family = AF_INET;
    localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
    localAddr.sin_port = htons(0);
  
    rc = bind(socket, (struct sockaddr *) &localAddr, sizeof(localAddr));
    if(rc<0) {
        cerr << "cannot bind port TCP " << dialog.get_port() << endl;
        perror("error ");
        return;
    }
				
    // connect to server 
    rc = ::connect(socket, (struct sockaddr *) &servAddr, sizeof(servAddr));
    if(rc<0) {
        perror("cannot connect ");
        return;
    }

    send(socket,"WELCOME SERVER\n",15,0);

    char buffer[512];

    read(socket, buffer, 512);

    if(!strncmp("WELCOME OBSERVER\n",buffer,17)){
        cout << "Connected" << endl;
    }else{
        cout << "Cannot understand server" << endl << endl;
    }
    
    connected = true;
    update_actions();    
}

void
CoreInterface::disconnect()
{   
    if(!connected){
        return;
    }
    
    send(socket,"CDISCONNECT\n",12,0);

    close(socket);
    connected=false;
    socket=-1;
    update_actions();
}

Glib::RefPtr < Gtk::ActionGroup >
CoreInterface::get_actions()
{
    if(!m_refActionGroup){
        m_refActionGroup = Gtk::ActionGroup::create();
        m_refActionGroup->add(Gtk::Action::create("Connect",_("Connect to Server") ), sigc::mem_fun(*this, &CoreInterface::connect));
        m_refActionGroup->add(Gtk::Action::create("Disconnect",_("Disconnect from Server")), sigc::mem_fun(*this, &CoreInterface::disconnect));
        update_actions();
    }   
    return m_refActionGroup;
};


void 
CoreInterface::update_actions()
{
    Glib::RefPtr<Gtk::Action> connect = 
        m_refActionGroup->get_action("Connect");
    Glib::RefPtr<Gtk::Action> disconnect = 
        m_refActionGroup->get_action("Disconnect");
    
    if(connected){
        connect->set_sensitive(false);
        disconnect->set_sensitive(true);       
    }else{
        connect->set_sensitive(true);
        disconnect->set_sensitive(false);   
    }    
}

bool
CoreInterface::send_command(const char command,
                            Glib::ustring& string)
{
    Glib::ustring message = "C";
    message+=command;
    message+=" ";
    message+=string;
    message+='\n';
      
    size_t result = send(socket,message.c_str(),message.length(),0);
    
    if(result == message.length()){
        return true;
    }else{
        cerr << "unable to send message" << endl;
        return false;
    }
}

bool
CoreInterface::start_tournament(Glib::ustring& file)
{
    if(!connected){
        return false;
    }
    return send_command('N', file);
}

ConnectDialog::ConnectDialog()
 : Gtk::Dialog(_("Connect to server"),false,false),
   m_optionsbox(_("Connect to server"))
{
    set_border_width(12);
    add_button(_("Connect"), Gtk::RESPONSE_OK);
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    
    get_vbox()->set_spacing(12);
    get_vbox()->set_homogeneous(false);
    get_vbox()->pack_start(m_optionsbox,false,false,0);
    
    
    m_optionsbox.add_option(_("Server:"),m_server);
    m_server.set_text("localhost");
   
    m_optionsbox.add_option(_("Port:"),m_port);
    m_port.set_range(1,65535);
    m_port.set_increments(1,100);
    m_port.set_value(32134);
  
    show_all();
}

Glib::ustring
ConnectDialog::get_server()
{
    return m_server.get_text();
}
    
int
ConnectDialog::get_port()
{
    return int(m_port.get_value());
}
