/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "MainWindow.h"

#include "NewTournament.h"
#include "CoreInterface.h"
#include "About.h"

#include <gtkmm/stock.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/filechooserdialog.h>

#include <glib/gi18n.h>
#include <sstream>

Glib::ustring ui_info =
    "<ui>"
    "  <menubar name='MenuBar'>"
    "    <menu action='FileMenu'>"
    "      <menuitem action='Conectar'/>"
    "	   <menuitem action='Disconnect'/>"   
    "      <separator/>"
    "      <menuitem action='Quit'/>"
    "    </menu>"
    "    <menu action='ViewMenu'>"
    "      <menuitem action='ZoomIn'/>"
    "      <menuitem action='ZoomOut'/>"
    "      <menuitem action='Zoom100'/>"
    "    </menu>"
    "    <menu action='HelpMenu'>"
    "      <menuitem action='About'/>"
    "    </menu>"
    "  </menubar>"
    "  <toolbar name='ToolBar'>"
    "    <toolitem action='ZoomIn'/>"
    "    <toolitem action='ZoomOut'/>"
    "    <toolitem action='Zoom100'/>"
    "  </toolbar>" "</ui>";


using namespace std;

MessageView *the_messageview;
ScoreView *the_scoreview;
ArenaView *the_arenaview;

MainWindow::MainWindow()
 : Gtk::Window(Gtk::WINDOW_TOPLEVEL)
{
    set_title("RealTimeBattle Client");

    the_arenaview = new ArenaView;
    the_messageview = new MessageView;
    the_scoreview = new ScoreView;

    add(m_Box);

    m_refActionGroup = Gtk::ActionGroup::create();
      
    // file menu
    m_refActionGroup->add(Gtk::Action::create("FileMenu", _("File")));
    m_refActionGroup->add(Gtk::Action::create("Conectar", Gtk::Stock::NEW, _("Connect to server")), sigc::mem_fun(*this, &MainWindow::new_tournament));

    m_refActionGroup->add(Gtk::Action::create("Quit", Gtk::Stock::QUIT), sigc::mem_fun(*this, &MainWindow::quit_rtb));

    // view menu
    m_refActionGroup->add(Gtk::Action::create("ViewMenu", _("View")));
    
    // help menu
    m_refActionGroup->add(Gtk::Action::create("HelpMenu", _("Help")));
    m_refActionGroup->add(Gtk::Action::create("About", _("About")), sigc::ptr_fun(&About::show_dialog));


    m_refUIManager = Gtk::UIManager::create();
    m_refUIManager->insert_action_group(m_refActionGroup);
    m_refUIManager->insert_action_group(the_arenaview->get_actions());
    m_refUIManager->insert_action_group(the_iface.get_actions());
    add_accel_group(m_refUIManager->get_accel_group());

    try {
        m_refUIManager->add_ui_from_string(ui_info);
    } catch(const Glib::Error & ex) {
        g_warning("building menus failed: %s", ex.what().c_str());
    }

    Gtk::Widget * menubar = m_refUIManager->get_widget("/MenuBar");
    if (menubar) {
        m_Box.pack_start(*menubar, Gtk::PACK_SHRINK);
    }
    Gtk::Widget * toolbar = m_refUIManager->get_widget("/ToolBar");
    if (toolbar) {
        m_Box.pack_start(*toolbar, Gtk::PACK_SHRINK);
    }

    m_VPaned.set_flags(Gtk::CAN_FOCUS);
    m_VPaned.set_position(0);
    m_VPaned.pack1(*the_scoreview);
    m_VPaned.pack2(*the_messageview);

    m_HPaned.set_flags(Gtk::CAN_FOCUS);
    m_HPaned.set_position(0);
    m_HPaned.pack1(m_VPaned);
    m_HPaned.pack2(*the_arenaview);

    m_ProgressBar.set_fraction(0.0);
    m_StatusBar.pack_start(m_ProgressBar);
    m_StatusBar.pack_start(m_TurnInfo);

    m_StatusBar.push(tournament_file);

    m_Box.pack_start(m_HPaned);
    m_Box.pack_start(m_StatusBar, Gtk::PACK_SHRINK);

    show_all_children(); 
    
}


MainWindow::~MainWindow()
{

}

void
MainWindow::new_tournament()
{
    //NewTournament newdialog;

    //int result = newdialog.run();

   /* if(result==Gtk::RESPONSE_OK){
	    if (newdialog.write_tournament_file() == false) {
            Gtk::MessageDialog dialog(_("Error starting tournament"), FALSE, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
            dialog.run();
	    } else { */
	    
            //tournament_file = newdialog.get_tournament_file();
            
            the_iface.connect();
            if(!the_iface.is_connected()){
                cerr << "Unable to connect to server" << endl;
                return;;
            } 
            
          //  if(the_arenaview->start(tournament_file.data(),RTB_CORE) == false) {
	    if(the_arenaview->start("H",RTB_CORE) == false) {
                Gtk::MessageDialog dialog(_("Error opening tournament"), FALSE, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
                dialog.run();
            }
           
           // the_iface.start_tournament(tournament_file);
	  //  }
	//}
}

void
MainWindow::open_logfile(const char* file)
{
    if (the_arenaview->is_started == true) {
        Gtk::MessageDialog dialog(_("This action will kill the current tournament.\n" "Do you want to do that?"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO);
        if (dialog.run() == Gtk::RESPONSE_NO)
	    return;
    }
    
    if(!file){
        Gtk::FileChooserDialog dialog(_("Please choose a tournament logfile"), Gtk::FILE_CHOOSER_ACTION_OPEN);
        dialog.set_transient_for(*this);

        //Add response buttons the the dialog:
        dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
        dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

        int result = dialog.run();

        if (result != Gtk::RESPONSE_OK){
            return;
        }
        tournament_file = dialog.get_filename();
    }else{
        tournament_file=file;
    }
        
    the_messageview->clear();
    the_scoreview->clear();

    if (the_arenaview->start(tournament_file.c_str(), RTB_LOGFILE) == false) {
        Gtk::MessageDialog dialog(_("Error opening tournament"), FALSE, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
        dialog.run();
    }
    m_StatusBar.push(tournament_file);
}

void
MainWindow::end_clicked()
{
}

void
MainWindow::pause()
{
    the_arenaview->pause();
}

void
MainWindow::quit_rtb()
{
    hide();
}

void
MainWindow::options_clicked()
{
}

void
MainWindow::statistics_clicked()
{
}

void
MainWindow::start_test_tournament(const char *data)
{
    open_logfile(data);
}

void
MainWindow::set_progress(double time, double timeout)
{
    double progress;
    
    if(time<=0 || timeout<=0){
        progress=0;
    }else{
        progress=time/timeout;
    }
        
    if (progress > 1){
        progress = 1;
    }
    
    m_ProgressBar.set_fraction(progress);

    if(timeout>0){
        int time_left = int(timeout - time);        
        if(time_left>=0){  
            stringstream tmp(ios::in | ios::out | ios::trunc);
            tmp << time_left << " of " << int(timeout) << " seconds left";
            m_ProgressBar.set_text(tmp.str());
        }            
    }        
}

void
MainWindow::set_gamesequence(const int game, const int seq, const int games, const int seqs)
{
    stringstream tmp(ios::in | ios::out | ios::trunc);
    tmp << "Game " << game << "(" << games << ") Seq " << seq << "(" << seqs << ")";
    m_TurnInfo.set_text(tmp.str());
}
