/* 
 * RealTimeBattle Client
 * Copyright (C) 2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "NewTournament.h"

#include "Stuff.h"

#include <gtkmm/filechooserdialog.h>
#include <gtkmm/stock.h>
#include <gtkmm/notebook.h>
#include <gtkmm/treerowreference.h>
#include <gtkmm/radiobuttongroup.h>
#include <glib/gi18n.h>

#include <iostream>
#include <fstream>
using namespace std;


static string robotsdir = "/usr/games/RealTimeBattle/Robots/";
static string arenasdir = "/usr/games/RealTimeBattle/Arenas/";

NewTournament::NewTournament()
 : Gtk::Dialog(_("Start new tournament"),false,false)
{
    set_border_width(12);
    get_vbox()->set_spacing(12);
    
    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
       
    Gtk::Notebook *notebook = manage(new Gtk::Notebook);
    get_vbox()->pack_start(*notebook);
    
    Gtk::VBox *vbox;
    Gtk::HBox *hbox;
    PP::OptionsBox *options;
    Glib::RefPtr<Gtk::SizeGroup> sizegroup;
    
    // tournament tab
    vbox = manage(new Gtk::VBox(false,18));
    vbox->set_border_width(12);
    notebook->append_page(*vbox,_("Tournament"));
    
    options = manage(new PP::OptionsBox(_("General")));
    vbox->pack_start(*options,false,false,0);
    sizegroup = options->get_size_group1();
    options->add_option(_("Games per sequence"),m_gps);
    m_gps.set_range(1,999);
    m_gps.set_increments(1,2);
    m_gps.set_value(1);
    
    options->add_option(_("Robots per sequence"),m_rps);
    m_rps.set_range(2,999);
    m_rps.set_increments(1,2);
    m_rps.set_value(2);
    
    options->add_option(_("Number of sequences"),m_nos);
    m_nos.set_range(1,999);
    m_nos.set_increments(1,2);
    m_nos.set_value(1);
    
    
    options = manage(new PP::OptionsBox(_("Robot defaults")));
    options->set_size_group1(sizegroup);
    vbox->pack_start(*options,false,false,0);
        
    options->add_option(_("Start energy"),m_energy);
    m_energy.set_range(1,999);
    m_energy.set_increments(1,25);
    m_energy.set_value(100);
        
    options->add_option(_("Max energy"),m_max_energy);
    m_max_energy.set_range(1,999);
    m_max_energy.set_increments(1,25);
    m_max_energy.set_value(120);
        
    
    options = manage(new PP::OptionsBox(_("Load/Save tournament")));
    options->set_size_group1(sizegroup);
    vbox->pack_start(*options,false,false,0);
    
    Gtk::Button *button;
    button = manage(new Gtk::Button(Gtk::Stock::OPEN));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::load_tournament));
    options->add_option(_("Open tournament file"),*button);
    
    button = manage(new Gtk::Button(Gtk::Stock::SAVE));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::save_tournament));
    options->add_option(_("Save tournament to file"),*button);

    //robots tab
    vbox = manage(new Gtk::VBox(false,18));
    vbox->set_border_width(12);
    notebook->append_page(*vbox,_("Robots"));
    
    options = manage(new PP::OptionsBox(_("Select robots for tournament")));
    vbox->pack_start(*options,false,true,0);
   
    hbox = manage(new Gtk::HBox(false,12));
    options->add_option(*hbox);
    
    Gtk::ScrolledWindow *scrolled_window;
    scrolled_window = manage(new Gtk::ScrolledWindow);
        
    m_ref_sr_model = Gtk::ListStore::create(m_Columns);
    m_selected_robots.set_model(m_ref_sr_model);
    m_selected_robots.append_column(_("Selected robots"), m_Columns.name);
    m_selected_robots.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);

    scrolled_window->add(m_selected_robots);
    scrolled_window->set_size_request(170, 200);
    hbox->pack_start(*scrolled_window,true,true,0);
    
    Gtk::VButtonBox *buttonbox;
    buttonbox = manage(new Gtk::VButtonBox(Gtk::BUTTONBOX_START,12));
    
    button = manage(new Gtk::Button(Gtk::Stock::ADD));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_robots));
    buttonbox->pack_start(*button,false,false,0);
    
    button = manage(new Gtk::Button(Gtk::Stock::REMOVE));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::remove_robots));
    buttonbox->pack_start(*button,false,false,0);
    hbox->pack_start(*buttonbox,false,false,0);
    
    scrolled_window = manage(new Gtk::ScrolledWindow);
    m_ref_ar_model = Gtk::ListStore::create(m_Columns);
    m_available_robots.set_model(m_ref_ar_model);
    m_available_robots.append_column(_("Available robots"), m_Columns.name);
    m_available_robots.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
    scrolled_window->add(m_available_robots);
    scrolled_window->set_size_request(170, 200);
    hbox->pack_start(*scrolled_window,true,true,0);
    read_dir_to_list(robotsdir, m_ref_ar_model);


    options = manage(new PP::OptionsBox(_("Custom robot")));
    vbox->pack_start(*options,false,false,0);
    
    button = manage(new Gtk::Button(Gtk::Stock::OPEN));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_custom_robot));
    options->add_option(_("Add custom robot"),*button);
    
    //arenas tab
    vbox = manage(new Gtk::VBox(false,18));
    vbox->set_border_width(12);
    notebook->append_page(*vbox,_("Arenas"));
    
    options = manage(new PP::OptionsBox(_("Select arenas for tournament")));
    vbox->pack_start(*options,false,true,0);
   
    hbox = manage(new Gtk::HBox(false,12));
    options->add_option(*hbox);
    
    scrolled_window = manage(new Gtk::ScrolledWindow);
        
    m_ref_sa_model = Gtk::ListStore::create(m_Columns);
    m_selected_arenas.set_model(m_ref_sa_model);
    m_selected_arenas.append_column(_("Selected arenas"), m_Columns.name);
    m_selected_arenas.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);

    scrolled_window->add(m_selected_arenas);
    scrolled_window->set_size_request(170, 200);
    hbox->pack_start(*scrolled_window,true,true,0);
    
    buttonbox = manage(new Gtk::VButtonBox(Gtk::BUTTONBOX_START,12));
    
    button = manage(new Gtk::Button(Gtk::Stock::ADD));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_arenas));
    buttonbox->pack_start(*button,false,false,0);
    
    button = manage(new Gtk::Button(Gtk::Stock::REMOVE));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::remove_arenas));
    buttonbox->pack_start(*button,false,false,0);
    hbox->pack_start(*buttonbox,false,false,0);
    
    scrolled_window = manage(new Gtk::ScrolledWindow);
    m_ref_aa_model = Gtk::ListStore::create(m_Columns);
    m_available_arenas.set_model(m_ref_aa_model);
    m_available_arenas.append_column(_("Available arenas"), m_Columns.name);
    m_available_arenas.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
    scrolled_window->add(m_available_arenas);
    scrolled_window->set_size_request(170, 200);
    hbox->pack_start(*scrolled_window,true,true,0);
    
    read_dir_to_list(arenasdir, m_ref_aa_model, ".arena");


    options = manage(new PP::OptionsBox(_("Custom arena")));
    vbox->pack_start(*options,false,false,0);
    
    button = manage(new Gtk::Button(Gtk::Stock::OPEN));
    button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_custom_arena));
    options->add_option(_("Add custom arena"),*button);   
    
    //environment tab
    vbox = manage(new Gtk::VBox(false,18));
    vbox->set_border_width(12);
    notebook->append_page(*vbox,_("Environment"));
    
    
    Gtk::RadioButtonGroup radiogroup;
    options = manage(new PP::RadioOptionsBox(radiogroup, _("Select environment for tournament")));
    vbox->pack_start(*options,false,true,0);
    
    scrolled_window = manage(new Gtk::ScrolledWindow);
        
    m_ref_env_model = Gtk::ListStore::create(m_Columns);
    m_env_list.set_model(m_ref_env_model);
    m_env_list.append_column(_("Selected environment"), m_Columns.name);

    scrolled_window->add(m_env_list);
    scrolled_window->set_size_request(180, 200);
    hbox = manage(new Gtk::HBox(false,12));
    options->add_option(*hbox);
    hbox->pack_start(*scrolled_window,false,false,0),
        
    buttonbox = manage(new Gtk::VButtonBox(Gtk::BUTTONBOX_START,12));
    
    button = manage(new Gtk::Button(Gtk::Stock::NEW));
    //button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_robots));
    buttonbox->pack_start(*button,false,false,0);
 
   // button = manage(new Gtk::Button(Gtk::Stock::EDIT));
    //button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::remove_robots));
   // buttonbox->pack_start(*button,false,false,0); 
    
    button = manage(new Gtk::Button(Gtk::Stock::REMOVE));
    //button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::remove_robots));
    buttonbox->pack_start(*button,false,false,0);
    
    hbox->pack_start(*buttonbox,false,false,0);
    

    
    options = manage(new PP::RadioOptionsBox(radiogroup, _("Use custom environment for tournament")));
    vbox->pack_start(*options,false,true,0);

    button = manage(new Gtk::Button(Gtk::Stock::OPEN));
    //button->signal_clicked().connect(sigc::mem_fun(*this, &NewTournament::add_custom_arena));
    options->add_option(_("Open custom environment file"),*button); 
    
    
    // load last tournament if available
    tempfile = Glib::get_tmp_dir();
    tempfile += "/tmp.tour";
    read_tournament_file(tempfile);

    show_all_children();
}

NewTournament::~NewTournament()
{
    // TODO: put destructor code here
}

void
NewTournament::read_dir_to_list(Glib::ustring directory, Glib::RefPtr < Gtk::ListStore > &refTree, gchar * extension)
{
    Glib::Dir dir(directory);
    Glib::DirIterator it;

    for (it = dir.begin(); it != dir.end(); it++){
        if ((*it).find(extension) != -1) {
            Gtk::TreeModel::Row row;
            row = *(refTree->append());
            row[m_Columns.name] = Glib::filename_to_utf8(*it);
            row[m_Columns.file] = directory + Glib::filename_to_utf8(*it);
        }
    }
}

void
NewTournament::add_robot_callback(const Gtk::TreeModel::iterator & iter)
{
    Gtk::TreeModel::Row row = *iter;
    Gtk::TreeModel::Row dest_row;

    dest_row = *(m_ref_sr_model->append());
   
    dest_row[m_Columns.name] = Glib::ustring(row[m_Columns.name]);
    dest_row[m_Columns.file] = Glib::ustring(row[m_Columns.file]);
}

void
NewTournament::add_arena_callback(const Gtk::TreeModel::iterator & iter)
{
    Gtk::TreeModel::Row row = *iter;
    Gtk::TreeModel::Row dest_row;

    dest_row = *(m_ref_sa_model->append());
    dest_row[m_Columns.name] = Glib::ustring(row[m_Columns.name]);
    dest_row[m_Columns.file] = Glib::ustring(row[m_Columns.file]);
}

bool
NewTournament::write_tournament_file()
{
    return write_tournament_file(tempfile);
}

bool
NewTournament::write_tournament_file(string & filename)
{
    typedef Gtk::TreeModel::Children type_children;

    ofstream tfile;

    tfile.open(filename.c_str(), ios::trunc);


    int robots = 0;
    {
	tfile << "Robots:" << endl;
	type_children children = m_ref_sr_model->children();
	for (type_children::iterator iter = children.begin(); iter != children.end(); ++iter) {
	    Gtk::TreeModel::Row row = *iter;
	    tfile << row[m_Columns.file] << endl;
	    robots++;
	}
    }
    if (robots <= 0)
	return false;

    int arenas = 0;
    {
	tfile << "Arenas:" << endl;
	type_children children = m_ref_sa_model->children();
	for (type_children::iterator iter = children.begin(); iter != children.end(); ++iter) {
	    Gtk::TreeModel::Row row = *iter;
	    tfile << row[m_Columns.file] << endl;
	    arenas++;
	}
    }
    if (arenas <= 0)
	return false;


    tfile << "Robots/Sequence: " << int(m_rps.get_value()) << endl;
    tfile << "Games/Sequence: " << int(m_gps.get_value()) << endl;
    tfile << "Sequences: " << int(m_nos.get_value()) << endl;
    tfile.close();
    return true;
}

void
NewTournament::read_tournament_file(string &filename)
{
    enum{
        STATUS_START,
        STATUS_ROBOTS,
        STATUS_ARENAS
    }status = STATUS_START;

    char temp[256];

    fstream file(filename.c_str(), ios::in);

    if(!file.is_open()){
        cerr << "error opening tournament file" << endl;
        return;        
    }        
        
    m_ref_sr_model->clear();
    m_ref_sa_model->clear();

	while (!file.eof()) {
	    string op;
	    file >> op;
	    if (file.eof())
		break;
	    if (op == "Robots:"){
            status = STATUS_ROBOTS;
        }else if(op == "Arenas:"){
            status = STATUS_ARENAS;
        }else if(op == "Robots/Sequence:"){
            int value;
            file >> value;
            m_rps.set_value(value);
        }else if(op == "Games/Sequence:"){
            int value;
            file >> value;
            m_gps.set_value(value);
        }else if(op == "Sequences:"){
            int value;
            file >> value;
            m_nos.set_value(value);
        }else if(status == STATUS_ROBOTS){
            Gtk::TreeModel::Row row;
            row = *(m_ref_sr_model->append());
            row[m_Columns.name] = op;
            row[m_Columns.file] = op;
	    }else if(status == STATUS_ARENAS){
            Gtk::TreeModel::Row row;
            row = *(m_ref_sa_model->append());
            row[m_Columns.name] = op;
            row[m_Columns.file] = op;
	    }

	    file.getline(temp, 255);
	}
}

string
NewTournament::get_tournament_file()
{
    return tempfile;
}

void
NewTournament::load_tournament()
{
    Gtk::FileChooserDialog dialog(_("Open tournament file"), Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	int result = dialog.run();
	if (result == Gtk::RESPONSE_OK) {
	    string tournament_file = dialog.get_filename();
	    read_tournament_file(tournament_file);
	}    
}

void
NewTournament::save_tournament()
{
    Gtk::FileChooserDialog dialog(_("Save tournament to file"), Gtk::FILE_CHOOSER_ACTION_SAVE);
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);

	int result = dialog.run();
	if (result == Gtk::RESPONSE_OK) {
	    string tournament_file = dialog.get_filename();
	    write_tournament_file(tournament_file);
	}
};

void
NewTournament::add_robots()
{
    m_available_robots.get_selection()->selected_foreach_iter(sigc::mem_fun(*this, &NewTournament::add_robot_callback));
}

void
NewTournament::remove_robots()
{
    Glib::RefPtr<const Gtk::TreeSelection> selection;
    selection = m_selected_robots.get_selection();
    
    list< Gtk::TreeModel::Path > rows = selection->get_selected_rows();
    list< Gtk::TreeRowReference > refs;

    {
        list< Gtk::TreeModel::Path >::iterator it;
        for(it=rows.begin(); it!=rows.end(); it++){
            refs.push_back(Gtk::TreeRowReference(m_ref_sr_model,(*it)));
        }
    }
    {
        list< Gtk::TreeRowReference >::iterator it;
        for(it=refs.begin(); it!=refs.end(); it++){
            Gtk::TreeModel::iterator iter;
            iter = m_ref_sr_model->get_iter((*it).get_path());
            m_ref_sr_model->erase(iter);            
        }
    } 
}

void
NewTournament::add_custom_robot()
{
    Gtk::FileChooserDialog dialog(_("Add custom robot"), Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	int result = dialog.run();
	if (result == Gtk::RESPONSE_OK){
        string file = dialog.get_filename();
        Gtk::TreeModel::Row row;
        row = *(m_ref_sr_model->append());
        row[m_Columns.name] = file; 
        row[m_Columns.file] = file;
    }
}

void
NewTournament::add_arenas()
{
    m_available_arenas.get_selection()->selected_foreach_iter(sigc::mem_fun(*this, &NewTournament::add_arena_callback));
}

void
NewTournament::remove_arenas()
{
    Glib::RefPtr<const Gtk::TreeSelection> selection;
    selection = m_selected_arenas.get_selection();
    
    list< Gtk::TreeModel::Path > rows = selection->get_selected_rows();
    list< Gtk::TreeRowReference > refs;

    {
        list< Gtk::TreeModel::Path >::iterator it;
        for(it=rows.begin(); it!=rows.end(); it++){
            refs.push_back(Gtk::TreeRowReference(m_ref_sa_model,(*it)));
        }
    }
    {
        list< Gtk::TreeRowReference >::iterator it;
        for(it=refs.begin(); it!=refs.end(); it++){
            Gtk::TreeModel::iterator iter;
            iter = m_ref_sa_model->get_iter((*it).get_path());
            m_ref_sa_model->erase(iter);            
        }
    }
}

void
NewTournament::add_custom_arena()
{
    Gtk::FileChooserDialog dialog(_("Add custom arena"), Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

	int result = dialog.run();
	if (result == Gtk::RESPONSE_OK){
        string file = dialog.get_filename();
        Gtk::TreeModel::Row row;
        row = *(m_ref_sa_model->append());
        row[m_Columns.name] = file; 
        row[m_Columns.file] = file;
    }
}
