/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ScoreView.h"

#include <glib/gi18n.h>
#include <sstream>
using namespace std;


ScoreView::ScoreView()
{
    set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    set_size_request(250, 125);
    set_border_width(12);
    set_shadow_type(Gtk::SHADOW_IN);

    add(m_TreeView);

    //Create list
    m_refTreeModel = Gtk::ListStore::create(m_Columns);
    m_TreeView.set_model(m_refTreeModel);

    //TODO: Show the ID in Debug Mode
    //m_TreeView.append_column(_("ID"), m_Columns.m_col_id);

    m_TreeView.append_column("", m_Columns.m_col_color);
    m_TreeView.append_column(_("name"), m_Columns.m_col_name);
    m_TreeView.append_column(_("energy"), m_Columns.m_col_energy);
    m_TreeView.append_column(_("place"), m_Columns.m_col_place);

    m_TreeView.set_rules_hint();

    show_all();
}


ScoreView::~ScoreView()
{
}

void
 ScoreView::clear()
{
    m_refTreeModel->clear();
}


void ScoreView::add_robot(int id, char *name, long color)
{
    Gtk::TreeModel::Row row;
    row = *(m_refTreeModel->append());
    row[m_Columns.m_col_id] = id;

    Glib::RefPtr < Gdk::Pixbuf > robot_color = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB, false, 8, 4, 16);
    robot_color->fill(color << 8);

    row[m_Columns.m_col_color] = robot_color;
    row[m_Columns.m_col_name] = name;
}

void ScoreView::update_robot(int id, double energy, int place)
{
    typedef Gtk::TreeModel::Children type_children;
    type_children children = m_refTreeModel->children();
    for (type_children::iterator iter = children.begin(); iter != children.end(); ++iter) {
	Gtk::TreeModel::Row row = *iter;
	if (row[m_Columns.m_col_id] == id) {
	    row[m_Columns.m_col_energy] = int (energy);
	    if (place == 0) {
		row[m_Columns.m_col_place] = "";
	    } else if (place > 0) {
		stringstream tmp(ios::in | ios::out | ios::trunc);
		tmp << place;
		row[m_Columns.m_col_place] = tmp.str();
	    }
	}
    }
}
