/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Shapes.h"
#include "Arena.h"

extern Arena the_arena;


Shape::Shape()
{
    // TODO: put constructor code here
}


Shape::~Shape()
{
    // TODO: put destructor code here
}

Line::Line(double x1, double y1, double x2, double y2, double thickness)
{
    this->x1 = x1;
    this->y1 = y1;
    this->x2 = x2;
    this->y2 = y2;
    this->thickness = thickness;
}

void
 Line::draw_shape(ArenaDrawingArea * the_view)
{
    the_view->draw_line(x1, y1, x2, y2, thickness);
}

Polygon::Polygon(std::vector < Vector2D > points, double thickness)
{
    this->points = points;
    this->thickness = thickness;
}

void
 Polygon::draw_shape(ArenaDrawingArea * the_view)
{
    the_view->draw_lines(points, thickness);
}

Circle::Circle(double x, double y, double radius, bool fill, long color)
{
    this->x = x;
    this->y = y;
    this->radius = radius;
    this->fill = fill;
    this->color = color;
}

void
 Circle::draw_shape(ArenaDrawingArea * the_view)
{
    if (fill == false)
	the_view->draw_circle(x, y, radius, color, false, the_arena.scale);
    else
	the_view->draw_circle(x, y, radius, color);
}

Arc::Arc(double x, double y, double radius, double angle1, double angle2, double thickness)
{
    this->x = x;
    this->y = y;
    this->radius = radius;
    this->angle1 = angle1;
    this->angle2 = angle2;
    this->thickness = thickness;
}

void
 Arc::draw_shape(ArenaDrawingArea * the_view)
{
    the_view->draw_arc(x, y, radius, angle1, angle2, thickness);
}


MovingShape::MovingShape(double x, double y, long color)
{
    this->x = x / the_arena.scale;
    this->y = y / the_arena.scale;
    this->color = color;
}


void
 MovingShape::draw_shape(ArenaDrawingArea * the_view)
{
    this->the_view = the_view;
    the_view->draw_circle(x, y, 0.3 / the_arena.scale, color);
}

void MovingShape::remove_shape()
{
    the_view->draw_circle(x, y, 0.3 / the_arena.scale, 0xffffff);
}



Shot::Shot(double x, double y, Vector2D & direction)
{
    this->x = x / the_arena.scale;
    this->y = y / the_arena.scale;
    this->direction.x = direction.x / the_arena.scale;
    this->direction.y = direction.y / the_arena.scale;
    alive = false;
}

void
 Shot::draw_shape(ArenaDrawingArea * the_view)
{
    this->the_view = the_view;
    the_view->draw_circle(x, y, 0.1 / the_arena.scale, 0x000000);
    alive = true;
}

void Shot::update_shape()
{
    if (alive == true) {
	the_view->draw_circle(x, y, 0.1 / the_arena.scale, 0xffffff);
	x += direction.x * the_arena.timescale;
	y += direction.y * the_arena.timescale;
	the_view->draw_circle(x, y, 0.1 / the_arena.scale, 0x000000);
    }
}

void Shot::remove_shape()
{
    if (alive == true) {
	the_view->draw_circle(x, y, 0.1 / the_arena.scale, 0xffffff);
    }
}
