/* 
 * RealTimeBattle Client
 * Copyright (C) 2003-2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _SHAPES_H_
#define _SHAPES_H_

#include "Vector2D.h"
#include "ArenaDrawingArea.h"


class Shape {
  public:
    Shape();
    virtual ~ Shape();

    virtual void draw_shape(ArenaDrawingArea * the_view) = 0;

  protected:
    // Shapes variables

    // TODO: add member variables...

};


class Line:public Shape {
  public:
    Line(double x1, double y1, double x2, double y2, double thickness);

    virtual void draw_shape(ArenaDrawingArea * the_view);
  protected:
    double x1, y1, x2, y2, thickness;

};

class Polygon:public Shape {
  public:
    Polygon(std::vector < Vector2D > points, double thickness);

    virtual void draw_shape(ArenaDrawingArea * the_view);
  protected:
     std::vector < Vector2D > points;
    double thickness;

};

class Circle:public Shape {
  public:
    Circle(double x, double y, double radius, bool fill = true, long color = 0x000000);

    virtual void draw_shape(ArenaDrawingArea * the_view);
  protected:
    double x, y;
    double radius;
    double fill;
    long color;
};

class Arc:public Shape {
  public:
    Arc(double x, double y, double radius, double angle1, double angle2, double thickness);

    virtual void draw_shape(ArenaDrawingArea * the_view);
  protected:
    double x, y;
    double radius;
    double thickness;
    double angle1, angle2;
};

class MovingShape:public Shape {
  public:
    MovingShape(double x, double y, long color);

    virtual void draw_shape(ArenaDrawingArea * the_view);
    virtual void remove_shape();


  protected:
     ArenaDrawingArea * the_view;
    double x, y;
    long color;
};

class Shot:public Shape {
  public:
    Shot(double x, double y, Vector2D & direction);

    virtual void draw_shape(ArenaDrawingArea * the_view);
    virtual void remove_shape();
    virtual void update_shape();

  protected:
     bool alive;
    ArenaDrawingArea *the_view;
    double x, y;
    Vector2D direction;
};
#endif				//_SHAPES_H_
