/* 
 * RealTimeBattle Client
 * Copyright (C) 2005 Volker Stroebel <volker@planetpenguin.de>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Stuff.h"

namespace PP{
    
void
label_set_bold(Gtk::Label &label)
{
    Pango::AttrList attrs;
    Pango::Attribute attribute;
        
    attribute=Pango::Attribute::create_attr_weight(Pango::WEIGHT_BOLD);
    
    attribute.set_start_index(0);
    attribute.set_end_index(~0);
    
    attrs.insert(attribute);
    label.set_attributes(attrs);        
}

    
    
    
    
    
    
    
    
OptionsBox::OptionsBox()
 : VBox(false,12),
   m_content_box(false,12),
   m_separator_label("    ")
{
} 
    
OptionsBox::OptionsBox(Glib::ustring title)
 : VBox(false,12),
   m_content_box(false,12),
   m_separator_label("    ")
{
    m_title.set_text(title);    
    m_title.set_alignment(0.0,0.5);
    label_set_bold(m_title);
    pack_start(m_title);
    
    add_content_box();
}

void
OptionsBox::add_content_box()
{
    m_group1 = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
    m_group2 = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
            
    Gtk::HBox *hbox=manage(new Gtk::HBox(false,0));
    hbox->pack_start(m_separator_label,false,false,0);
    hbox->pack_start(m_content_box);
    pack_start(*hbox);
}

void
OptionsBox::add_option(Glib::ustring title, Gtk::Widget &widget)
{
    Gtk::HBox *hbox=manage(new Gtk::HBox(false,12));
    m_content_box.pack_start(*hbox,false,false,0);
    
    Gtk::Label *label = manage(new Gtk::Label);
    label->set_markup(title);
    label->set_alignment(0.0,0.5);
    
    hbox->pack_start(*label,false,false,0);
    m_group1->add_widget(*label);
    
    hbox->pack_start(widget,false,false,0);
    m_group2->add_widget(widget);
}

void
OptionsBox::add_option(Gtk::Widget &widget)
{
    m_content_box.pack_start(widget);
}

void
OptionsBox::set_size_group1(Glib::RefPtr<Gtk::SizeGroup> group)
{
    m_group1 = group;
}

void
OptionsBox::set_size_group2(Glib::RefPtr<Gtk::SizeGroup> group)
{
    m_group2=group;
}

Glib::RefPtr<Gtk::SizeGroup>
OptionsBox::get_size_group1()
{
    return m_group1;
}

Glib::RefPtr<Gtk::SizeGroup>
OptionsBox::get_size_group2()
{
    return m_group2;    
}

RadioOptionsBox::RadioOptionsBox(Gtk::RadioButtonGroup& group,
                                 Glib::ustring title)
 : m_radio(group)
{
    m_title.set_text(title);  
    m_title.set_alignment(0.0,0.5);
    label_set_bold(m_title);
    
    m_radio.add(m_title);
    m_radio.signal_toggled().connect(sigc::mem_fun(*this, &RadioOptionsBox::radio_toggled));
    pack_start(m_radio);
    m_content_box.set_sensitive(m_radio.get_active());
    
    add_content_box();
}

void
RadioOptionsBox::radio_toggled()
{
    m_content_box.set_sensitive(m_radio.get_active());
}

} // namespace PP
